<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 21/04/20
 * Time: 11.43
 */

namespace FiloBlu\Rma\Handlers;
use Magento\Rma\Model\Rma\Source\Status;
use FiloBlu\Rma\Api\RmaExtensionAttributesRepositoryInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use FiloBlu\Rma\Helper\ChangeSizeAttributeHelper;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Store\Model\ScopeInterface;
use FiloBlu\Rma\Api\Model\Config\Reader;
use FiloBlu\Rma\Helper\SerializerHelper;
use Magento\Rma\Model\Item\Attribute\Source\Status as ItemStatus;
use Magento\Rma\Api\RmaRepositoryInterface;
use FiloBlu\Rma\Model\Rma\Status\HistoryFactory;

class RmaItemStatusUpdateHandler
{

    const XML_PATH_RECEIVED_EMAIL_ENABLE = 'sales_email/magento_rma_receive/enabled';

    const XML_PATH_RMA_REASON_MAPPING = 'sales/rma_reason/reason';

    const XML_PATH_RMA_AUTO_AUTHORIZED = 'filoblu_disablerma_section/rma_group/rma_auto_authorized';


    /**
     * @var RmaExtensionAttributesRepositoryInterface
     */
    private $rmaExtensionAttributesRepository;
    /**
     * @var OrderRepositoryInterface
     */
    private $orderRepository;
    /**
     * @var ChangeSizeAttributeHelper
     */
    private $changeSizeAttributeHelper;
    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var Reader
     */
    private $reader;
    /**
     * @var SerializerHelper
     */
    private $serializer;
    /**
     * @var RmaRepositoryInterface
     */
    private $rmaRepository;
    /**
     * @var HistoryFactory
     */
    private $historyFactory;


    /**
     * RmaItemStatusUpdateHandler constructor.
     * @param RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository
     * @param OrderRepositoryInterface $orderRepository
     * @param ChangeSizeAttributeHelper $changeSizeAttributeHelper
     * @param ScopeConfigInterface $scopeConfig
     * @param Reader $reader
     * @param SerializerHelper $serializer
     * @param RmaRepositoryInterface $rmaRepository
     * @param HistoryFactory $historyFactory
     */
    public function __construct(
        RmaExtensionAttributesRepositoryInterface $rmaExtensionAttributesRepository,
        OrderRepositoryInterface $orderRepository,
        ChangeSizeAttributeHelper $changeSizeAttributeHelper,
        ScopeConfigInterface $scopeConfig,
        Reader $reader,
        SerializerHelper $serializer,
        RmaRepositoryInterface $rmaRepository,
        HistoryFactory $historyFactory){

        $this->rmaExtensionAttributesRepository = $rmaExtensionAttributesRepository;
        $this->orderRepository = $orderRepository;
        $this->changeSizeAttributeHelper = $changeSizeAttributeHelper;
        $this->scopeConfig = $scopeConfig;
        $this->reader = $reader;
        $this->serializer = $serializer;
        $this->rmaRepository = $rmaRepository;
        $this->historyFactory = $historyFactory;
    }


    /**
     * @param \Magento\Rma\Model\Rma $subject
     * @param $result
     */
    public function shouldUpdateStatus(\Magento\Rma\Model\Rma $subject, $result){

        switch (true) {

            case ($result->getStatus() == Status::STATE_PROCESSED_CLOSED || $result->getStatus() == Status::STATE_DENIED || $result->getStatus() == Status::STATE_CLOSED) :
                foreach ($subject->getItems() as $item){

                    $rma_item_id = $item->getEntityId();
                    $rmaId = $subject->getEntityId();
                    $changeSizeOrderId = $this->rmaExtensionAttributesRepository->getChangeSizeOrderId($rma_item_id);
                    if($changeSizeOrderId) {
                        $changeSizeOrder = $this->orderRepository->get($changeSizeOrderId);
                        $payment = $changeSizeOrder->getPayment()->getMethod();
                        if ($payment == \FiloBlu\Rma\Model\Payment\ChangeSizePay::PAYMENT_CODE) {
                            if ($item->getResolution() == $this->changeSizeAttributeHelper->getChangeSizeAttributeOptionId() and $item->getStatus() == Status::STATE_APPROVED) {
                                $changeSizeOrder = $this->orderRepository->get($changeSizeOrderId);

                                $changeSizeOrder->setStatus('payment_confirmed')->setState('processing');
                            }else {
                                $changeSizeOrder = $this->orderRepository->get($changeSizeOrderId);
                                $changeSizeOrder->setStatus('canceled')->setState('canceled');
                            }
                            $changeSizeOrder->addStatusHistoryComment("Status and State updated programatically by rma with id $rmaId update");
                            $changeSizeOrder->save();
                        }
                    }
                }
                break;

            case  ($result->getStatus() == Status::STATE_PENDING) :
                if($this->scopeConfig->getValue(self::XML_PATH_RMA_AUTO_AUTHORIZED,ScopeInterface::SCOPE_STORE))
                {
                    /** @var $result \Magento\Rma\Model\Rma */
                    $items = $result->getItems();
                    $standard_reasons_config = $this->reader->read();
                    $automatic_approval = array();
                    foreach($standard_reasons_config as $reason):
                        if($reason['automatic_approval']==1):
                            $automatic_approval[] = $reason['id'];
                        endif;
                    endforeach;
                    /** @var \Magento\Rma\Api\Data\ItemInterface $item */
                    $auth = 0;

                    $itemsComments ="";
                    foreach($items as $item):
                        $item_status = $item->getStatus();
                        $item_brand_reason = $item->getReason();
                        $mapping = $this->serializer->unserialize($this->scopeConfig->getValue(self::XML_PATH_RMA_REASON_MAPPING,ScopeInterface::SCOPE_STORE, $subject->getStoreId()));

                        foreach ($mapping as $reason):
                            if($item_brand_reason == $reason['brand_reason'] && in_array($reason['standard_reason'],$automatic_approval)):
                                // avanzamento dello stato dell'item
                                $item->setStatus(ItemStatus::STATE_AUTHORIZED);
                                $itemsComments  .=" -- ".$item->getProductAdminName()." automatically authorized -- ";
                                $item->setQtyAuthorized($item->getQtyRequested());
                                $item->save();
                                $auth++;
                                break;
                            endif;
                        endforeach;
                    endforeach;

                    if($auth > 0 && $auth==count($items)):
                        $this->rmaRepository->get($result->getEntityId())
                            ->setStatus(Status::STATE_AUTHORIZED)
                            ->save();
                    elseif($auth > 0):
                        $this->rmaRepository->get($result->getEntityId())
                            ->setStatus(Status::STATE_PARTIAL_AUTHORIZED)
                            ->save();
                    endif;

                    if($auth > 0){
                        $statusHistory = $this->historyFactory->create();
                        $statusHistory->setRmaEntityId($result->getEntityId());
                        $statusHistory->saveComment($itemsComments,false,true);
                    }

                    break;
                }

        }

    }
}